#!/usr/bin/python3.5
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: structure
#------------------------------------------------------------------------------
# Work structure for extraction processing. 
#==============================================================================

#==============================================================================
# Importing required modules
#==============================================================================

import glob
from rdflib import Graph
from lib.config import Config


#==============================================================================
# Parameters
#==============================================================================

# Config File
config_file = "config.xml"


#==============================================================================
# Graph Loading
#==============================================================================
    
def load_config(config, work_graph):
    print("-- Configuration Loading:")
    
    work_graph.parse(config.schema_file)
    print("----- RDF Schema (" + str(len(work_graph)) + ")")
    
    work_graph.parse(config.semantic_net_file)
    print("----- Semantic Net Definition (" + str(len(work_graph)) + ")")
    
    work_graph.parse(config.dash_file)
    print("----- Data Shapes Dash (" + str(len(work_graph)) + ")")
    
    work_graph.parse(config.config_param_file)
    print("----- Config Parameter Definition (" + str(len(work_graph)) + ")")

    
def load_cts(config, work_graph):
    print("-- CTS Loading:")
    
    work_graph.parse(config.cts_file)
    print("----- All Schemes (" + str(len(work_graph)) + ")")


def load_frame(config, work_graph):
    print("-- Frame Ontology Loading:")
        
    work_graph.parse(config.frame_ontology_file)
    print("----- System Frame Ontology (" + str(len(work_graph)) + ")")
    
    
def load_sentences(config, work_graph):
    print("-- Sentence Loading:")
    
    for file_ref in glob.glob(config.source_sentence_files, recursive = True):
        work_graph.parse(file_ref)
        print("----- " + file_ref + " (" + str(len(work_graph)) + ")") 
        
    

#==============================================================================
# Result (export)
#==============================================================================
     
def export_result(config, work_graph, export_file):
    print("-- Export result as turtle: " + export_file)
    
    work_graph.serialize(destination=export_file, 
                         base=config.base_uri + '/' + config.source_ref, 
                         format='turtle')
    
    
def finalize_export_file(export_file):
    """ finalize the export file by adding some useful prefixes """
    
    with open(export_file, "rt") as file:
        x = file.read()
    
    with open(export_file, "wt") as file:
        x = x.replace(
            "@prefix sys: <https://unsel.tetras-libre.fr/tenet/frame/system-ontology/> .",
            """
            @prefix sys: <https://unsel.tetras-libre.fr/tenet/frame/system-ontology/> .
            @prefix sys-Event: <https://unsel.tetras-libre.fr/tenet/frame/system-ontology/Event#> .
            @prefix sys-State_Property: <https://unsel.tetras-libre.fr/tenet/frame/system-ontology/State_Property#> .
            @prefix sys-abstract_thing: <https://unsel.tetras-libre.fr/tenet/frame/system-ontology/abstract_thing#> .
            @prefix sys-action_verb: <https://unsel.tetras-libre.fr/tenet/frame/system-ontology/action_verb#> .
            @prefix sys-agent: <https://unsel.tetras-libre.fr/tenet/frame/system-ontology/agent#> .
            @prefix sys-attributive_verb: <https://unsel.tetras-libre.fr/tenet/frame/system-ontology/attributive_verb#> .
            @prefix sys-component: <https://unsel.tetras-libre.fr/tenet/frame/system-ontology/component#> .
            @prefix sys-message: <https://unsel.tetras-libre.fr/tenet/frame/system-ontology/message#> .
            @prefix sys-place: <https://unsel.tetras-libre.fr/tenet/frame/system-ontology/place#> .
            @prefix owl: <http://www.w3.org/2002/07/owl#> .
            @prefix rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#> .
            @prefix rdfs: <http://www.w3.org/2000/01/rdf-schema#> .
            @prefix sys-relation: <https://unsel.tetras-libre.fr/tenet/frame/system-ontology/relation/> .
            @prefix xsd: <http://www.w3.org/2001/XMLSchema#> .
            """)
        file.write(x)
   
     
#==============================================================================
# Main Function
#==============================================================================
  
def create_extraction_graph(corpus, source_ref, target_ref):
    try:
        print("[Tenet] Prepare work data from corpus " + corpus)
    
        print("\n" + "- Config Loading")
        config = Config(config_file, corpus, source_ref, target_ref)
        print(config.__dict__)
    
        print("\n" + "- Graph Initialization")
        work_graph = Graph()
        load_config(config, work_graph)   
        load_frame(config, work_graph) 
    
        print("\n" + "- Preparation of Transduction Process")
        load_cts(config, work_graph)
    
        print("\n" + "- Data Source Imports")
        load_sentences(config, work_graph)
    
        print("\n" + "- Result")
        export_file = config.output_file
        export_result(config, work_graph, export_file)
        finalize_export_file(export_file)
        
        print()
    
    except:
        print("!!! An exception occurred !!!") 
    
